@@ -9,9 +9,11 @@
  *
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <string.h>
-#include "server.h"
+#include <libtest/server.h>
 
 int main(void)
 {
